#region Copyright Syncfusion Inc. 2001 - 2014
// Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Threading;
using Syncfusion.Windows.Shared;
using System.Diagnostics;

using Syncfusion.Data;
using Syncfusion.UI.Xaml.Grid;

namespace PerformanceDemo
{

    public class FastViewModel:NotificationObject
    {        
        GridVirtualizingCollectionView data;
        ObservableCollection<Tick> prints;
        public FastViewModel()
        {
            prints = new ObservableCollection<Tick>();
            data = new GridVirtualizingCollectionView(prints);

            Add(); Add(); Add();
        }

        public GridVirtualizingCollectionView Stocks
        {
            get { return data; }
        }

        public void  Add()
        {
            Tick mm = new Tick();
            mm.Time = DateTime.Now.Ticks.ToString();
            prints.Insert(0, mm);
        }

    }




    public class StocksViewModel : NotificationObject, IDisposable
    {
        ObservableCollection<StockData> data;
        Random r = new Random(123345345);
        internal DispatcherTimer timer;
        private bool enableTimer = false;
        private double timerValue;
        private int noOfUpdates = 1000;

        

        /// <summary>
        /// Gets the stocks.
        /// </summary>
        /// <value>The stocks.</value>
        public ObservableCollection<StockData> Stocks
        {
            get
            {
                return this.data;
            }
        }

        #region Constructor

        public void AddTick()
        {
            var newRec = new StockData();
            newRec.Column0 = data.Count.ToString();
            data.Insert(0, newRec);         
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StocksViewModel"/> class.
        /// </summary>
        public StocksViewModel()
        {
            this.data = new ObservableCollection<StockData>();
            //this.AddRows(400);
            this.timer = new DispatcherTimer();
            this.ResetRefreshFrequency(1000);
            timer.Interval = TimeSpan.FromMilliseconds(1000);
            timer.Tick += timer_Tick;
        }

        /// <summary>
        /// Sets the interval.
        /// </summary>
        /// <param name="time">The time.</param>
        public void SetInterval(TimeSpan time)
        {
            this.timer.Interval = time;
        }

        /// <summary>
        /// Starts the timer.
        /// </summary>
        /// 
        public void StartTimer()
        {
            if (!this.timer.IsEnabled)
            {
                this.timer.Start();
            }
        }

        /// <summary>
        /// Gets or sets the timer value.
        /// </summary>
        /// <value>The timer value.</value>
        public double TimeSpanValue
        {
            get
            {
                return timerValue;
            }
            set
            {
                timerValue = value;
                this.timer.Interval = TimeSpan.FromMilliseconds(timerValue);
                RaisePropertyChanged("TimeSpanValue");
            }
        }

        /// <summary>
        /// Stops the timer.
        /// </summary>
        public void StopTimer()
        {
            this.timer.Stop();
        }

        /// <summary>
        /// Handles the Tick event of the timer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void timer_Tick(object sender, object e)
        {
            int startTime = DateTime.Now.Millisecond;
            Stopwatch t = new Stopwatch();
            t.Start();
            ChangeRows(noOfUpdates);
            t.Stop();
            Debug.WriteLine(t.Elapsed.Milliseconds.ToString());

        }

        /// <summary>
        /// Gets or sets a value indicating whether [enable timer].
        /// </summary>
        /// <value><c>true</c> if [enable timer]; otherwise, <c>false</c>.</value>
        public bool EnableTimer
        {
            get
            {
                return this.enableTimer;
            }
            set
            {
                this.enableTimer = value;
            }
        }

        /// <summary>
        /// Adds the rows.
        /// </summary>
        /// <param name="count">The count.</param>
        private void AddRows(int count)
        {
            for (int i = 0; i < count; ++i)
            {
                var newRec = new StockData();
                double d = r.NextDouble();
                newRec.Column1 = Math.Round(r.NextDouble() * 30, 2);
                newRec.Column2 = r.Next();
                newRec.Column3 = Math.Round(r.NextDouble() * 10, 2);
                newRec.Column4 = r.Next();
                newRec.Column5 = r.Next();
                newRec.Column6= r.Next();
                newRec.Column7 = Math.Round((1 + r.NextDouble() * 30));
                newRec.Column8 = Math.Round((1 + r.NextDouble() * 5));
                newRec.Column9 = r.Next();
                newRec.Column10 = r.Next();
                newRec.Column11 = Math.Round((1 + r.NextDouble() * 30));
                newRec.Column12= r.Next();
                newRec.Column13= r.Next();
                newRec.Column14= r.Next();
                newRec.Column15 = Math.Round((1 + r.NextDouble() * 2));
                newRec.Column16= r.Next();
                newRec.Column17= r.Next();
                newRec.Column18= r.Next();
                newRec.Column19 = Math.Round((1 + r.NextDouble() * 20));
                newRec.Column20= r.Next();
                newRec.Column21= r.Next();
                newRec.Column22 = Math.Round((1 + r.NextDouble() * 10));
                newRec.Column23= r.Next();
                newRec.Column24= r.Next();
                newRec.Column25= r.Next();
                newRec.Column26 = Math.Round((1 + r.NextDouble() * 30));
                newRec.Column27 = r.Next();
                newRec.Column28 = r.Next();
                newRec.Column29 = r.Next();
                newRec.Column30 = Math.Round((1 + r.NextDouble() * 5));
                newRec.Column31 = r.Next();
                newRec.Column32 = r.Next();
                newRec.Column33 = r.Next();
                newRec.Column34 = Math.Round((1 + r.NextDouble() * 20));
                newRec.Column35 = r.Next();
                newRec.Column36 = r.Next();
                newRec.Column37 = Math.Round((1 + r.NextDouble() * 50));
                newRec.Column38 = r.Next();
                newRec.Column39 = r.Next();
                newRec.Column40 = r.Next();                                
                data.Add(newRec);
            }
        }

        /// <summary>
        /// Resets the refresh frequency.
        /// </summary>
        /// <param name="changesPerTick">The changes per tick.</param>
        public void ResetRefreshFrequency(int changesPerTick)
        {
            if (this.timer == null)
            {
                return;
            }

            if (!this.noOfUpdates.Equals(changesPerTick))
            {
                this.StopTimer();
                this.noOfUpdates = changesPerTick;
                if (enableTimer)
                    this.StartTimer();
            }
        }

        /// <summary>
        /// Changes the rows.
        /// </summary>
        /// <param name="count">The count.</param>
        public void ChangeRows(int count)
        {
            //if (data.Count > count)
            //    count = data.Count;
            for (int i = 0; i < count; ++i)
            {
                int recNo = r.Next(data.Count);
                StockData recRow = data[recNo];
                double d = r.NextDouble();
                if (d < .5)
                {
                    data[recNo].Column1 = Math.Round((1 + r.NextDouble() * -5));
                    data[recNo].Column3 = Math.Round((1 + r.NextDouble() *10));
                    data[recNo].Column7 = Math.Round((1 + r.NextDouble() * 50));
                    data[recNo].Column11 = Math.Round((1 + r.NextDouble() * -3));
                    data[recNo].Column15 = Math.Round((1 + r.NextDouble() * 20));
                    data[recNo].Column19 = Math.Round((1 + r.NextDouble() * 10));
                    data[recNo].Column22 = Math.Round((2 + r.NextDouble() * -5));
                    data[recNo].Column27 = Math.Round((1 + r.NextDouble() * 20));
                    data[recNo].Column33 = Math.Round((1 + r.NextDouble() * 10));
                    data[recNo].Column39 = Math.Round((2 + r.NextDouble() * -5));                    
                }
                else
                {
                    data[recNo].Column1 =  Math.Round(d, 2) * -5;
                    data[recNo].Column3 =  Math.Round(d, 2) * -3;
                    data[recNo].Column7 =  Math.Round(d, 2) * -1;
                    data[recNo].Column11 = Math.Round(d, 1) * -3;
                    data[recNo].Column15 = Math.Round(d, 1) * -2;
                    data[recNo].Column19 = Math.Round(d, 1) * -3;
                    data[recNo].Column22 = Math.Round(d, 2) * -4;
                    data[recNo].Column27 = Math.Round(d, 2) * -5;
                    data[recNo].Column33 = Math.Round(d, 2) * -3;
                    data[recNo].Column39 = Math.Round(d, 2) * -1;                    
                } 
            }
        }


        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            if (this.timer != null)
            {
                this.timer.Tick -= timer_Tick;
                this.StopTimer();
            }
        }

        #endregion
    }
}
